/*++========================================================================
 * Program Name:     NetWare Redirector for Linux
 * File Name:        commands.h
 * Version:          v1.00
 * Author:           James Turner/Richard Williams
 *
 * Abstract:         This modules contains all defined commands.
 *
 * Notes:
 * Revision History: 
 *
 *
 * Copyright (C) 2005 Novell, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *======================================================================--*/

/*===[ Include files specific to Linux ]==================================*/

/*===[ Include files specific to this module ]============================*/

/*===[ External data ]====================================================*/

/*===[ External prototypes ]==============================================*/

/*===[ Manifest constants ]===============================================*/
#ifndef _COMMANDS_H
#define _COMMANDS_H

#define VFS_COMMAND_GET_CONNECTED_SERVER_LIST   0
#define VFS_COMMAND_GET_SERVER_VOLUME_LIST      1
#define VFS_COMMAND_VERIFY_FILE                 2
#define VFS_COMMAND_OPEN_CONNECTION_BY_ADDR     3
#define VFS_COMMAND_LOGIN_IDENTITY              4
#define VFS_COMMAND_ENUMERATE_DIRECTORY         5
#define VFS_COMMAND_OPEN_FILE                   6
#define VFS_COMMAND_CREATE_FILE                 7
#define VFS_COMMAND_CLOSE_FILE                  8
#define VFS_COMMAND_READ_FILE                   9
#define VFS_COMMAND_WRITE_FILE                  10
#define VFS_COMMAND_DELETE_FILE                 11
#define VFS_COMMAND_CREATE_DIRECOTRY            12
#define VFS_COMMAND_START_ENUMERATE             13
#define VFS_COMMAND_END_ENUMERATE               14
#define VFS_COMMAND_LOGIN_USER                  15
#define VFS_COMMAND_LOGOUT_USER                 16
#define VFS_COMMAND_CREATE_CONTEXT              17
#define VFS_COMMAND_DESTROY_CONTEXT             18
#define VFS_COMMAND_SET_FILE_INFO               19
#define VFS_COMMAND_TRUNCATE_FILE               20
#define VFS_COMMAND_OPEN_CONNECTION_BY_NAME     21
#define VFS_COMMAND_XPLAT_CALL                  22
#define VFS_COMMAND_RENAME_FILE                 23
#define VFS_COMMAND_ENUMERATE_DIRECTORY_EX      24
#define VFS_COMMAND_GETPWUD                     25
#define VFS_COMMAND_ENUM_XCONN                  26
#define VFS_COMMAND_READ_STREAM                 27
#define VFS_COMMAND_WRITE_STREAM                28
#define VFS_COMMAND_CLOSE_STREAM                29
#define VFS_COMMAND_GET_VERSION                 30
#define VFS_COMMAND_SET_MOUNT_PATH              31
#define VFS_COMMAND_GET_USER_SPACE              32
#define VFS_COMMAND_DBG                         33


#define  NWD_ACCESS_QUERY                        0x00000001
#define  NWD_ACCESS_READ                         0x00000002
#define  NWD_ACCESS_WRITE                        0x00000004
#define  NWD_ACCESS_EXECUTE                      0x00000008
#define  NWD_ACCESS_VALID                        0x0000000F

/*
   Share Mode

   A value of zero in a shared mode field specifies the caller
   desires exclusive access to the object.
*/

#define  NWD_SHARE_READ                          0x00000001
#define  NWD_SHARE_WRITE                         0x00000002
#define  NWD_SHARE_DELETE                        0x00000004
#define  NWD_SHARE_VALID                         0x00000007

/*
   Creates a new file.  The create API will fail if the specified
   file already exists.
*/
#define  NWD_DISP_CREATE_NEW                     0x00000001

/*
   Creates a new file.  If the specified file already exists,
   the create API will overwrite the old file and clear the
   existing attributes.
*/
#define  NWD_DISP_CREATE_ALWAYS                  0x00000002

/*
   Opens the file.  The API will fail if the file does not exist.
*/
#define  NWD_DISP_OPEN_EXISTING                  0x00000003

/*
   Opens the file.  If the file does not exist, the API will
   create the file.
*/
#define  NWD_DISP_OPEN_ALWAYS                    0x00000004

/*
   Opens the file.  When the file is opened the API will truncate
   the stream to zero bytes.  The API will fail if the file
   does not exist.
*/
#define  NWD_DISP_TRUNCATE_EXISTING              0x00000005
#define  NWD_DISP_MAXIMUM                        0x00000005

/*
   Open/Create returned information values

   The bottom two bytes of NWD_ACTION are returned
   as a value.  All values are mutually exclusive.
*/

#define  NWD_ACTION_OPENED                       0x00000001
#define  NWD_ACTION_CREATED                      0x00000002


#define  MAX_IO_SIZE							(1024 * 32)

/*===[ Type definitions ]=================================================*/

/*===[ Function prototypes ]==============================================*/

#pragma pack(push, 1)

#ifndef NWHANDLE
typedef void * NWHANDLE;
#endif

typedef struct _ncl_string
{
	unsigned int  	type;
	unsigned char 	*buffer;
	unsigned long	len;

} NclString, *PNclString;

typedef struct _nwd_string
{
	unsigned int	type;
	unsigned long	len;
	unsigned long	boffset;

} NwdString, *PNwdString;

typedef struct _COMMAND_REQUEST_HEADER
{
	unsigned long CommandType;
	unsigned long SequenceNumber;
	unsigned long long SessionId;

} COMMAND_REQUEST_HEADER, *PCOMMAND_REQUEST_HEADER;

typedef struct _COMMAND_REPLY_HEADER
{
	unsigned long Sequence_Number;
	unsigned long ErrorCode;

} COMMAND_REPLY_HEADER, *PCOMMAND_REPLY_HEADER;

typedef struct _CLOSE_REQUEST {
	COMMAND_REQUEST_HEADER Command;
	NWHANDLE FileHandle;
} CLOSE_REQUEST, *PCLOSE_REQUEST;

typedef struct _CLOSE_REPLY {
	COMMAND_REPLY_HEADER Reply;
} CLOSE_REPLY, *PCLOSE_REPLY;

typedef struct _DELETE_FILE_REQUEST {
	COMMAND_REQUEST_HEADER Command;
	unsigned long isDirectory;
	unsigned long pathlength;
	unsigned char path[1];
} DELETE_FILE_REQUEST, *PDELETE_FILE_REQUEST;

typedef struct _DELETE_FILE_REPLY {
	COMMAND_REPLY_HEADER Reply;
} DELETE_FILE_REPLY, *PDELETE_FILE_REPLY;

typedef struct _FLUSH_REQUEST {
	COMMAND_REQUEST_HEADER Command;
	NWHANDLE FileHandle;
} FLUSH_REQUEST, *PFLUSH_REQUEST;

typedef struct _FLUSH_REPLY {
	COMMAND_REPLY_HEADER Reply;
} FLUSH_REPLY, *PFLUSH_REPLY;

typedef struct _GET_FILEINFO_REQUEST {
	COMMAND_REQUEST_HEADER Command;
	NWHANDLE FileHandle;
} GET_FILEINFO_REQUEST, *PGET_FILEINFO_REQUEST;

typedef struct _GET_FILEINFO_REPLY {
	COMMAND_REPLY_HEADER Reply;
} GET_FILEINFO_REPLY, *PGET_FILEINFO_REPLY;

typedef struct _GET_CONNECTED_SERVER_LIST_REQUEST
{
	COMMAND_REQUEST_HEADER Command;
} GET_CONNECTED_SERVER_LIST_REQUEST, *PGET_CONNECTED_SERVER_LIST_REQUEST;

typedef struct _GET_CONNECTED_SERVER_LIST_REPLY
{
	COMMAND_REPLY_HEADER Reply;
	unsigned char List[1];
} GET_CONNECTED_SERVER_LIST_REPLY, *PGET_CONNECTED_SERVER_LIST_REPLY;


typedef struct _GET_CONNECTED_SERVER_LIST_REQUEST_EX
{
	COMMAND_REQUEST_HEADER Command;
} GET_CONNECTED_SERVER_LIST_REQUEST_EX, *PGET_CONNECTED_SERVER_LIST_REQUEST_EX;

typedef struct _GET_CONNECTED_SERVER_LIST_REPLY_EX
{
	COMMAND_REPLY_HEADER Reply;
	unsigned long	bufferLen;
	unsigned char 	List[1];

} GET_CONNECTED_SERVER_LIST_REPLY_EX, *PGET_CONNECTED_SERVER_LIST_REPLY_EX;

typedef struct _GET_SERVER_VOLUME_LIST_REQUEST
{
	COMMAND_REQUEST_HEADER Command;
	unsigned long Length;
	unsigned char Name[1];
} GET_SERVER_VOLUME_LIST_REQUEST, *PGET_SERVER_VOLUME_LIST_REQUEST;

typedef struct _GET_SERVER_VOLUME_LIST_REPLY
{
	COMMAND_REPLY_HEADER Reply;
	unsigned char List[1];
} GET_SERVER_VOLUME_LIST_REPLY, *PGET_SERVER_VOLUME_LIST_REPLY;

typedef struct _OPEN_CONNECTION_BY_ADDR_REQUEST
{
	COMMAND_REQUEST_HEADER Command;
	unsigned long 	address;

} OPEN_CONNECTION_BY_ADDR_REQUEST, *POPEN_CONNECTION_BY_ADDR_REQUEST;

typedef struct _OPEN_CONNECTION_BY_ADDR_REPLY
{
	COMMAND_REPLY_HEADER 	Reply;
	unsigned char			serverName[64];
	unsigned char			treeName[64];
	NWHANDLE				connHandle;

} OPEN_CONNECTION_BY_ADDR_REPLY, *POPEN_CONNECTION_BY_ADDR_REPLY;


typedef struct _OPEN_CONNECTION_BY_NAME_REQUEST
{
	COMMAND_REQUEST_HEADER Command;
	unsigned long	NameLen;
	unsigned char 	Name[1];

} OPEN_CONNECTION_BY_NAME_REQUEST, *POPEN_CONNECTION_BY_NAME_REQUEST;

typedef struct _OPEN_CONNECTION_BY_NAME_REPLY
{
	COMMAND_REPLY_HEADER 	Reply;
	unsigned char			treeName[64];
	NWHANDLE				connHandle;

} OPEN_CONNECTION_BY_NAME_REPLY, *POPEN_CONNECTION_BY_NAME_REPLY;

/*
typedef struct _LOGIN_IDENTITY_REQUEST
{
	COMMAND_REQUEST_HEADER Command;
	unsigned int			treeFlags;
	unsigned char			treeName[64];
	unsigned int			serverFlags;
	unsigned char			serverName[64];
	unsigned int			userFlags;
	unsigned char			userName[512];
	unsigned int			passwordFlags;
	unsigned char			password[128];

} LOGIN_IDENTITY_REQUEST, *PLOGIN_IDENTITY_REQUEST;

typedef struct _LOGIN_IDENTITY_REPLY
{
	COMMAND_REPLY_HEADER Reply;
	unsigned char			serverName[64];
	unsigned char			treeName[64];
	NWHANDLE					connHandle;

} LOGIN_IDENTITY_REPLY, *PLOGIN_IDENTITY_REPLY;
*/

typedef struct _VERIFY_FILE_REQUEST
{
	COMMAND_REQUEST_HEADER Command;
	unsigned long	pathLen;
	unsigned char	path[1];

} VERIFY_FILE_REQUEST, *PVERIFY_FILE_REQUEST;

typedef struct _VERIFY_FILE_REPLY
{
	COMMAND_REPLY_HEADER 	Reply;
	unsigned long			lastAccessTime;
	unsigned long			modifyTime;
	unsigned long			createTime;
	unsigned long long		fileSize;
	unsigned long 			fileMode;

} VERIFY_FILE_REPLY, *PVERIFY_FILE_REPLY;

typedef struct _BEGIN_ENUMERATE_DIRECTORY_REQUEST
{
	COMMAND_REQUEST_HEADER Command;
	unsigned long			pathLen;
	unsigned char			path[1];
		
} BEGIN_ENUMERATE_DIRECTORY_REQUEST, *PBEGIN_ENUMERATE_DIRECTORY_REQUEST;

typedef struct _BEGIN_ENUMERATE_DIRECTORY_REPLY
{
	COMMAND_REPLY_HEADER 	Reply;
	unsigned long			enumerateHandle;
		
} BEGIN_ENUMERATE_DIRECTORY_REPLY, *PBEGIN_ENUMERATE_DIRECTORY_REPLY;


typedef struct _END_ENUMERATE_DIRECTORY_REQUEST
{
	COMMAND_REQUEST_HEADER Command;
	unsigned long			enumerateHandle;
		
} END_ENUMERATE_DIRECTORY_REQUEST, *PEND_ENUMERATE_DIRECTORY_REQUEST;

typedef struct _END_ENUMERATE_DIRECTORY_REPLY
{
	COMMAND_REPLY_HEADER 	Reply;
		
} END_ENUMERATE_DIRECTORY_REPLY, *PEND_ENUMERATE_DIRECTORY_REPLY;

typedef struct _ENUMERATE_DIRECTORY_REQUEST
{
	COMMAND_REQUEST_HEADER Command;
	unsigned long			enumerateHandle;
	unsigned long			pathLen;
	unsigned char			path[1];
		
} ENUMERATE_DIRECTORY_REQUEST, *PENUMERATE_DIRECTORY_REQUEST;


typedef struct _ENUMERATE_DIRECTORY_REPLY
{
	COMMAND_REPLY_HEADER 	Reply;
	unsigned long			enumerateHandle;
	unsigned long			lastAccessTime;
	unsigned long			modifyTime;
	unsigned long			createTime;
	unsigned long long   size;
	unsigned long 			mode;
	unsigned long			nameLen;
	unsigned char			name[1];
		
} ENUMERATE_DIRECTORY_REPLY, *PENUMERATE_DIRECTORY_REPLY;

typedef struct _ENUMERATE_DIRECTORY_EX_REQUEST
{
	COMMAND_REQUEST_HEADER Command;
	unsigned long			enumerateHandle;
	unsigned long			pathLen;
	unsigned char			path[1];
		
} ENUMERATE_DIRECTORY_EX_REQUEST, *PENUMERATE_DIRECTORY_EX_REQUEST;


typedef struct _ENUMERATE_DIRECTORY_EX_DATA
{
	unsigned long			length;
	unsigned long			lastAccessTime;
	unsigned long			modifyTime;
	unsigned long			createTime;
	unsigned long long   size;
	unsigned long 			mode;
	unsigned long			nameLen;
	unsigned char			name[1];

} ENUMERATE_DIRECTORY_EX_DATA, *PENUMERATE_DIRECTORY_EX_DATA;

typedef struct _ENUMERATE_DIRECTORY_EX_REPLY
{
	COMMAND_REPLY_HEADER 	Reply;
	unsigned long			enumerateHandle;
	unsigned long 			enumCount;
		
} ENUMERATE_DIRECTORY_EX_REPLY, *PENUMERATE_DIRECTORY_EX_REPLY;



typedef struct _OPEN_FILE_REQUEST
{
	COMMAND_REQUEST_HEADER Command;
	unsigned long			access;  /* File Access */
	unsigned long			mode;    /* Sharing Mode */
	unsigned long			disp;    /* Create Disposition */
	unsigned long			pathLen;
	unsigned char			path[1];
		
} OPEN_FILE_REQUEST, *POPEN_FILE_REQUEST;


typedef struct _OPEN_FILE_REPLY
{
	COMMAND_REPLY_HEADER 	Reply;
	unsigned long			handle;
	unsigned long			lastAccessTime;
	unsigned long			modifyTime;
	unsigned long			createTime;
	unsigned long 			attributes;
	loff_t					size;
		
} OPEN_FILE_REPLY, *POPEN_FILE_REPLY;


typedef struct _CREATE_FILE_REQUEST 
{

	COMMAND_REQUEST_HEADER Command;
	unsigned long pathlength;
	unsigned char path[1];

} CREATE_FILE_REQUEST, *PCREATE_FILE_REQUEST;

typedef struct _CREATE_FILE_REPLY 
{
	COMMAND_REPLY_HEADER 	Reply;

} CREATE_FILE_REPLY, *PCREATE_FILE_REPLY;

typedef struct _CLOSE_FILE_REQUEST
{
	COMMAND_REQUEST_HEADER 	Command;
	unsigned long			handle;
		
} CLOSE_FILE_REQUEST, *PCLOSE_FILE_REQUEST;


typedef struct _CLOSE_FILE_REPLY
{
	COMMAND_REPLY_HEADER 	Reply;
		
} CLOSE_FILE_REPLY, *PCLOSE_FILE_REPLY;


typedef struct _READ_FILE_REQUEST
{
	COMMAND_REQUEST_HEADER 	Command;
	unsigned long			handle;
	loff_t					offset;
	size_t					len;
		
} READ_FILE_REQUEST, *PREAD_FILE_REQUEST;

typedef struct _READ_FILE_REPLY
{
	COMMAND_REPLY_HEADER 	Reply;
	unsigned long long 		bytesRead;
	unsigned char			data[1];
		
} READ_FILE_REPLY, *PREAD_FILE_REPLY;


typedef struct _WRITE_FILE_REQUEST
{
	COMMAND_REQUEST_HEADER 	Command;
	unsigned long			handle;
	loff_t					offset;
	size_t					len;
	unsigned char			data[1];
		
} WRITE_FILE_REQUEST, *PWRITE_FILE_REQUEST;

typedef struct _WRITE_FILE_REPLY
{
	COMMAND_REPLY_HEADER 	Reply;
	unsigned long long 		bytesWritten;
} WRITE_FILE_REPLY, *PWRITE_FILE_REPLY;

typedef struct _READ_STREAM_REQUEST
{
	COMMAND_REQUEST_HEADER  Command;
   unsigned long           connection;
	unsigned char           handle[6];
	loff_t                  offset;
	size_t                  len;
} READ_STREAM_REQUEST, *PREAD_STREAM_REQUEST;

typedef struct _READ_STREAM_REPLY
{
	COMMAND_REPLY_HEADER    Reply;
	size_t                  bytesRead;
	unsigned char           data[1];
} READ_STREAM_REPLY, *PREAD_STREAM_REPLY;


typedef struct _WRITE_STREAM_REQUEST
{
	COMMAND_REQUEST_HEADER  Command;
   unsigned long           connection;
	unsigned char           handle[6];
	loff_t                  offset;
	size_t                  len;
	unsigned char           data[1];
} WRITE_STREAM_REQUEST, *PWRITE_STREAM_REQUEST;

typedef struct _WRITE_STREAM_REPLY
{
	COMMAND_REPLY_HEADER    Reply;
	size_t                  bytesWritten;
} WRITE_STREAM_REPLY, *PWRITE_STREAM_REPLY;

typedef struct _CLOSE_STREAM_REQUEST
{
	COMMAND_REQUEST_HEADER 	Command;
   unsigned long           connection;
	unsigned char           handle[6];
} CLOSE_STREAM_REQUEST, *PCLOSE_STREAM_REQUEST;


typedef struct _CLOSE_STREAM_REPLY
{
	COMMAND_REPLY_HEADER 	Reply;
		
} CLOSE_STREAM_REPLY, *PCLOSE_STREAM_REPLY;

typedef struct _CREATE_DIRECTORY_REQUEST 
{

	COMMAND_REQUEST_HEADER Command;
	unsigned long pathlength;
	unsigned char path[1];

} CREATE_DIRECTORY_REQUEST, *PCREATE_DIRECTORY_REQUEST;

typedef struct _CREATE_DIRECTORY_REPLY 
{
	COMMAND_REPLY_HEADER 	Reply;

} CREATE_DIRECTORY_REPLY, *PCREATE_DIRECTORY_REPLY;

typedef struct _LOGIN_USER_REQUEST
{
	COMMAND_REQUEST_HEADER Command;
	unsigned long  srvNameType;
	unsigned long  serverLength;
	unsigned long  serverOffset;
	unsigned long  usrNameType;
	unsigned long  userNameLength;
	unsigned long  userNameOffset;
	unsigned long  pwdNameType;
	unsigned long  passwordLength;
	unsigned long  passwordOffset;

} LOGIN_USER_REQUEST, *PLOGIN_USER_REQUEST;

typedef struct _LOGIN_USER_REPLY
{
	COMMAND_REPLY_HEADER 	Reply;
	unsigned long           connectionHandle;
	unsigned long			loginIdentity;

} LOGIN_USER_REPLY, *PLOGIN_USER_REPLY;


typedef struct _LOGOUT_REQUEST
{
	COMMAND_REQUEST_HEADER Command;
	unsigned long  Length;
	unsigned char  Name[1];

} LOGOUT_REQUEST, *PLOGOUT_REQUEST;

typedef struct _LOGOUT_REPLY
{
	COMMAND_REPLY_HEADER 	Reply;

} LOGOUT_REPLY, *PLOGOUT_REPLY;


typedef struct _CREATE_CONTEXT_REQUEST
{
	COMMAND_REQUEST_HEADER Command;

} CREATE_CONTEXT_REQUEST, *PCREATE_CONTEXT_REQUEST;

typedef struct _CREATE_CONTEXT_REPLY
{
	COMMAND_REPLY_HEADER 	Reply;
	unsigned long long      SessionId;

} CREATE_CONTEXT_REPLY, *PCREATE_CONTEXT_REPLY;

typedef struct _DESTROY_CONTEXT_REQUEST
{
	COMMAND_REQUEST_HEADER  Command;

} DESTROY_CONTEXT_REQUEST, *PDESTROY_CONTEXT_REQUEST;

typedef struct _DESTROY_CONTEXT_REPLY
{
	COMMAND_REPLY_HEADER 	Reply;

} DESTROY_CONTEXT_REPLY, *PDESTROY_CONTEXT_REPLY;

/*
 * Attribute flags.  These should be or-ed together to figure out what
 * has been changed!
 */
#ifndef ATTR_MODE
#define ATTR_MODE	1
#define ATTR_UID	2
#define ATTR_GID	4
#define ATTR_SIZE	8
#define ATTR_ATIME	16
#define ATTR_MTIME	32
#define ATTR_CTIME	64
#define ATTR_ATIME_SET	128
#define ATTR_MTIME_SET	256
#define ATTR_FORCE	512	/* Not a change, but a change it */
#define ATTR_ATTR_FLAG	1024
#endif

typedef struct _LNX_FILE_INFO
{
	unsigned int 			ia_valid;
	unsigned int			ia_mode;
	uid_t					ia_uid;
	gid_t					ia_gid;
	loff_t					ia_size;
	time_t					ia_atime;
	time_t					ia_mtime;
	time_t					ia_ctime;
	unsigned int			ia_attr_flags;

} LX_FILE_INFO, *PLX_FILE_INFO;

typedef struct _SET_FILE_INFO_REQUEST
{
	COMMAND_REQUEST_HEADER Command;
	LX_FILE_INFO			fileInfo;
	unsigned int			pathlength;
	char					path[1];

} SET_FILE_INFO_REQUEST, *PSET_FILE_INFO_REQUEST;

typedef struct _SET_FILE_INFO_REPLY
{
	COMMAND_REPLY_HEADER 	Reply;

} SET_FILE_INFO_REPLY, *PSET_FILE_INFO_REPLY;


typedef struct _TRUNCATE_FILE_REQUEST
{
	COMMAND_REQUEST_HEADER 	Command;
	unsigned int			pathLen;
	char					path[1];

} TRUNCATE_FILE_REQUEST, *PTRUNCATE_FILE_REQUEST;

typedef struct _TRUNCATE_FILE_REPLY
{
	COMMAND_REPLY_HEADER 	Reply;

} TRUNCATE_FILE_REPLY, *PTRUNCATE_FILE_REPLY;

typedef struct _GETPWUID_REQUEST
{
	COMMAND_REQUEST_HEADER 	Command;
	unsigned int            uid;
} GETPWUID_REQUEST, *PGETPWUID_REQUEST;

typedef struct _GETPWUID_REPLY
{
	COMMAND_REPLY_HEADER 	Reply;
	unsigned char           UserName[1];
} GETPWUID_REPLY, *PGETPWUID_REPLY;

typedef struct _GET_VERSION_REQUEST
{
	COMMAND_REQUEST_HEADER 	Command;
} GET_VERSION_REQUEST, *PGET_VERSION_REQUEST;

typedef struct _GET_VERSION_REPLY
{
	COMMAND_REPLY_HEADER 	Reply;
	unsigned char           Version[1];
} GET_VERSION_REPLY, *PGET_VERSION_REPLY;

typedef struct _SET_MOUNT_PATH
{
	COMMAND_REQUEST_HEADER 	Command;
   unsigned long           PathLength;
	unsigned char           Path[1];
} SET_MOUNT_PATH_REQUEST, *PSET_MOUNT_PATH_REQUEST;

typedef struct _SET_MOUNT_PATH_REPLY
{
	COMMAND_REPLY_HEADER 	Reply;
} SET_MOUNT_PATH, *PSET_MOUNT_PATH_REPLY;

typedef struct _GET_USER_SPACE
{
	COMMAND_REQUEST_HEADER 	Command;
} GET_USER_SPACE_REQUEST, *PGET_USER_SPACE_REQUEST;

typedef struct _GET_USER_SPACE_REPLY
{
   COMMAND_REPLY_HEADER    Reply;
   uint64_t                TotalSpace;
   uint64_t                FreeSpace;
   uint64_t                TotalEnties;
   uint64_t                FreeEnties;
} GET_USER_SPACE_REPLY, *PGET_USER_SPACE_REPLY;

typedef struct _XPLAT_CALL_REQUEST
{
	COMMAND_REQUEST_HEADER 	Command;
	unsigned int			NwcCommand;
	unsigned long			dataLen;
	unsigned char			data[1];

} XPLAT_CALL_REQUEST, *PXPLAT_CALL_REQUEST;

typedef struct _XPLAT_CALL_REPLY
{
	COMMAND_REPLY_HEADER 	Reply;
	unsigned long			dataLen;
	unsigned char			data[1];

} XPLAT_CALL_REPLY, *PXPLAT_CALL_REPLY;


/* XPlat NWC structures used by the daemon */

typedef struct _NWD_OPEN_CONN_BY_NAME
{
	unsigned long	ConnHandle;
	unsigned long	nameLen;
	unsigned long	oName;			/* Ofset to the Name */
	unsigned long 	serviceLen;
	unsigned long	oServiceType;	/* Offset to service Type; */
	unsigned long 	uConnFlags;
	unsigned long	uTranType;
	unsigned long	newConnHandle;

} NwdCOpenConnByName, *PNwdCOpenConnByName;

typedef  struct   _NWD_TRAN_ADDR
{
   unsigned long	uTransportType;
   unsigned long	uAddressLength;
   unsigned long	oAddress;

} NwdCTranAddr, *PNwdCTranAddr;


typedef  struct   _NWD_OPEN_CONN_BY_ADDR
{
   unsigned long   	ConnHandle;
   unsigned long	oServiceType;
   unsigned long	uConnFlags;
   NwdCTranAddr		TranAddr;

} NwdCOpenConnByAddr, *PNwdCOpenConnByAddr;

typedef  struct   _NWD_CLOSE_CONN
{
   unsigned long	ConnHandle;

} NwdCCloseConn, *PNwdCCloseConn;

typedef struct _NWD_NCP_REQ
{
	unsigned long	ConnHandle;
	unsigned long	replyLen;
	unsigned long	requestLen;
	unsigned long	function;
/*	unsigned long 	subFunction; */
/*	unsigned long 	verb; */
	unsigned long	flags;
	unsigned char 	data[1];

} NwdCNCPReq, *PNwdCNCPReq;

typedef struct _NWD_NCP_REP
{
	unsigned long replyLen;
	unsigned char data[1];

} NwdCNCPRep, *PNwdCNCPRep;

typedef struct _NWC_AUTH_WID
{
	unsigned long  ConnHandle;
	unsigned long  AuthenticationId;

} NwdCAuthenticateWithId, *PNwdCAuthenticateWithId;

typedef  struct   _NWC_AUTHENTICATE
{
       unsigned long		ConnHandle;
       unsigned long        uAuthenticationType;
       unsigned long		userNameOffset;
       unsigned long		passwordOffset;
       unsigned long        MaxInfoLength;
       unsigned long        InfoLength;
       unsigned long		authenInfoOffset;

} NwdCAuthenticate, *PNwdCAuthenticate;

typedef  struct   _NWC_UNAUTHENTICATE
{
         unsigned long ConnHandle;
         unsigned long AuthenticationId;

} NwdCUnauthenticate, *PNwdCUnauthenticate;

typedef  struct   _NWC_LISC_ID
{
	unsigned long	ConnHandle;

} NwdCLicenseConn, *PNwdCLicenseConn;

typedef  struct   _NWC_UNLIC_CONN
{
	unsigned long	ConnHandle;

} NwdCUnlicenseConn, *PNwdCUnlicenseConn;

typedef  struct   _NWC_GET_IDENT_INFO
{
        unsigned long        AuthenticationId;
        unsigned long        AuthType;
        unsigned long        NameType;
        unsigned short int   ObjectType;
        unsigned long        IdentityFlags;
		unsigned long		 domainLen;
        unsigned long        pDomainNameOffset;
		unsigned long		 objectLen;
        unsigned long 		 pObjectNameOffset;

} NwdCGetIdentityInfo, *PNwdCGetIdentityInfo;


typedef  struct   _NWC_LO_ID
{
         unsigned long	AuthenticationId;

} NwdCLogoutIdentity, *PNwdCLogoutIdentity;

typedef struct _RENAME_FILE_REQUEST
{
	COMMAND_REQUEST_HEADER 	Command;
   int                  directoryFlag;
	unsigned long			newnameLen;
	unsigned char			newname[256];
	unsigned long			oldnameLen;
	unsigned char			oldname[256];
} RENAME_FILE_REQUEST, *PRENAME_FILE_REQUEST;

typedef struct _RENAME_FILE_REPLY
{
	COMMAND_REPLY_HEADER 	Reply;

} RENAME_FILE_REPLY, *PRENAME_FILE_REPLY;

typedef  struct   __NwdServerVersion
{
   unsigned long                   uMajorVersion;
   unsigned short int              uMinorVersion;
   unsigned short int              uRevision;

} NwdServerVersion, *PNwdServerVersion;


#define	MAX_ADDRESS_LENGTH	32

typedef  struct   tagNwdTranAddrEx
{
   unsigned long	uTransportType;
   unsigned long	uAddressLength;
   unsigned char	Buffer[MAX_ADDRESS_LENGTH];

} NwdTranAddr, *PNwdTranAddr;

typedef  struct   __NWD_CONN_INFO
{
   unsigned long      uInfoVersion;
   unsigned long      uAuthenticationState;
   unsigned long      uBroadcastState;
   unsigned long      uConnectionReference;
   unsigned long      pTreeNameOffset;
/*   unsigned long      pWorkGroupIdOffset;  Not used */
   unsigned long      uSecurityState;
   unsigned long      uConnectionNumber;
   unsigned long      uUserId;
   unsigned long      pServerNameOffset;
   unsigned long      uNdsState;
   unsigned long      uMaxPacketSize;
   unsigned long      uLicenseState;
   unsigned long      uPublicState;
   unsigned long      bcastState;
   unsigned long      pServiceTypeOffset;
   unsigned long      uDistance;
   unsigned long      uAuthId;
   unsigned long      uDisconnected;
   NwdServerVersion   ServerVersion;
   NwdTranAddr        TranAddress;

} NwdConnInfo, *PNwdConnInfo;


typedef struct _nwd_conn_info
{
	unsigned long 	ConnHandle;
    unsigned long   uInfoLevel;
    unsigned long   uInfoLength;

} NwdCGetConnInfo, *PNwdCGetConnInfo;


typedef  struct   nwd_open_conn_by_Ref
{
         unsigned long  uConnReference;
         unsigned long  uConnFlags;
         unsigned long	ConnHandle;

} NwdCOpenConnByRef, *PNwdCOpenConnByRef;

typedef  struct   nwd_get_reqversion
{
         unsigned long                uMajorVersion;
         unsigned long                uMinorVersion;
         unsigned long                uRevision;

} NwdCGetRequesterVersion, *PNwdCGetRequesterVersion;

typedef  struct   _nwc_scan_conn_info
{
     unsigned long        uScanIndex;
     unsigned long        uScanInfoLevel;
     unsigned long        uScanInfoLen;
     unsigned long        uScanConnInfoOffset;
     unsigned long        uScanFlags;
     unsigned long        uReturnInfoLevel;
     unsigned long        uReturnInfoLength;
     unsigned long        uConnectionReference;
	 unsigned long        uReturnConnInfoOffset;

} NwdCScanConnInfo, *PNwdCScanConnInfo;


typedef  struct   nwc_get_pref_ds_tree
{
      unsigned long       uTreeLength;
      unsigned long       DsTreeNameOffset;

} NwdCGetPreferredDsTree, *PNwdCGetPreferredDsTree;

typedef  struct   nwc_set_pref_ds_tree
{
      unsigned long       uTreeLength;
      unsigned long       DsTreeNameOffset;

} NwdCSetPreferredDsTree, *PNwdCSetPreferredDsTree;

typedef  struct   nwc_set_def_name_ctx
{
       unsigned long        uTreeLength;
       unsigned long		TreeOffset;
       unsigned long        uNameLength;
       unsigned long		NameContextOffset;

} NwdCSetDefaultNameContext, *PNwdCSetDefaultNameContext;

typedef  struct   nwc_get_def_name_ctx
{
       unsigned long        uTreeLength;
       unsigned long		TreeOffset;
       unsigned long        uNameLength;
       unsigned long        NameContextOffset;

} NwdCGetDefaultNameContext, *PNwdCGetDefaultNameContext;


typedef  struct   _nwc_get_treemonitored_connref
{
      NwdString			TreeName;
      unsigned long     uConnReference;

} NwdCGetTreeMonitoredConnRef, *PNwdCGetTreeMonitoredConnRef;

typedef  struct   _nwc_enumerate_identities
{
        unsigned long        Iterator;
		unsigned long		 domainNameLen;
        unsigned long        domainNameOffset;
        unsigned long        AuthType;
		unsigned long		 objectNameLen;
        unsigned long        objectNameOffset;
        unsigned long        NameType;
        unsigned short int   ObjectType;
        unsigned long        IdentityFlags;
        unsigned long        AuthenticationId;

} NwdCDEnumerateIdentities, *PNwdCEnumerateIdentities;

typedef  struct   nwd_change_key
{
         unsigned long        	domainNameOffset;
		 unsigned long			domainNameLen;
         unsigned long        	AuthType;
         unsigned long          objectNameOffset;
		 unsigned long			objectNameLen;
         unsigned long        	NameType;
         unsigned short int   	ObjectType;
         unsigned long          verifyPasswordOffset;
         unsigned long 			verifyPasswordLen;
         unsigned long			newPasswordOffset;
         unsigned long          newPasswordLen;

} NwdCChangeKey, *PNwdCChangeKey;

typedef  struct   _nwd_get_primary_conn
{
       unsigned long    uConnReference;

} NwdCGetPrimaryConnection, *PNwdCGetPrimaryConnection;

typedef  struct   _nwd_set_primary_conn
{
         unsigned long	ConnHandle;

} NwdCSetPrimaryConnection, *PNwdCSetPrimaryConnection;

typedef  struct   _nwd_map_drive_ex
{
          unsigned long         ConnHandle;
          unsigned long         localUid;
          unsigned long         linkOffsetLength;
          unsigned long         linkOffset;
          unsigned long         dirPathOffsetLength;
          unsigned long         dirPathOffset;

} NwdCMapDriveEx, *PNwdCMapDriveEx;

typedef  struct   _nwd_unmap_drive_ex
{
    unsigned long      linkLen;
    char               linkPath[1];

} NwdCUnmapDriveEx, *PNwdCUnmapDriveEx;


typedef struct  _nwd_enum_links
{
    unsigned long       totalLen;
    unsigned long       linkCount;
    
} NwdCEnumLinks, *PNwdCEnumLinks;

typedef  struct   nwd_getbroadcastnotification
{
        unsigned long       uMessageFlags;
        unsigned long       uConnReference;
        unsigned long       messageLen;
        char                message[1];

} NwdCGetBroadcastNotification, *PNwdCGetBroadcastNotification;


typedef struct _enum_entry
{
    unsigned long       entryLen;
    unsigned long       connHdl;
    char                data[1];
} NwdCEnumEntry, *PNwdCEnumEntry;

typedef  struct   _nwd_set_conn_info
{
        unsigned long       ConnHandle;
        unsigned long       uInfoLevel;
        unsigned long       uInfoLength;
        unsigned long       offsetConnInfo;

} NwdCSetConnInfo, *PNwdCSetConnInfo;

typedef struct _len_string
{
    unsigned long       stLen;
    char                string[1];
    
} LString, *PLString;


typedef struct _DEBUG_REQUEST
{
	COMMAND_REQUEST_HEADER 	Command;
    int                     cmdlen;
    char                    dbgcmd[1];
    
} DEBUG_REQUEST, *PDEBUG_REQUEST;

typedef struct _DEBUG_REPLY
{
	COMMAND_REPLY_HEADER 	Reply;

} DEBUG_REPLY, *PDEBUG_REPLY;


typedef  struct   _Nwd_Set_Key
{
         unsigned long          ConnHandle;
         unsigned long          AuthenticationId;
         unsigned long          objectNameLen;
         unsigned long          objectNameOffset;
         unsigned short int     ObjectType;
         unsigned long          newPasswordLen;
         unsigned long          newPasswordOffset;

} NwdCSetKey, *PNwdCSetKey;


typedef  struct   _Nwd_Verify_Key
{
        unsigned long       AuthType;
        unsigned long       NameType;
        unsigned short int  ObjectType;
        unsigned long       domainNameLen;
        unsigned long       domainNameOffset;
        unsigned long       objectNameLen;
        unsigned long       objectNameOffset;
        unsigned long       verifyPasswordLen;
        unsigned long       verifyPasswordOffset;         

} NwdCVerifyKey, *PNwdCVerifyKey;


#pragma pack(pop)

#endif
